class DebugException(Exception):
    pass


def assert_test(value, message, error=None, passed=None):
    if message is not None:
        print(message, end="...")
    if value:
        if passed is None:
            print("passed")
        else:
            print(passed)
    else:
        if error is None:
            print("FAILED!!")
        else:
            print(error)
        assert False


def exception_test(message,  exception, method_to_test, **kwargs):
    print(message, end="...")
    try:
        method_to_test(**kwargs)
        print()
        raise DebugException("{} test should have failed".format(message))
    except exception as e:
        if isinstance(e, DebugException):
            raise e
        pass
    print("passed")


def _test(throws, ex):
    if throws:
        raise ex("Throws")


class TestException(Exception):
    def __init__(self, message="Test Exception", errors=None):
        super().__init__(message)
        self.errors = errors


def main():
    exception_test("testing - should throw 0", Exception, _test, throws=True, ex=Exception)
    try:
        exception_test("testing - should throw because the function didn't throw", TestException,
                       _test, throws=False, ex=TestException)
        raise Exception("this should not pass!")
    except DebugException as e:
        pass
    try:
        exception_test("testing - should throw because exceptions don't match", Exception,
                       _test, throws=False, ex=TestException)
        raise Exception("this should not pass!")
    except DebugException as e:
        pass
    assert_test(True, "just some testing 0")
    assert_test(True, "just some testing 1", "error occurred 1")
    assert_test(True, "just some testing 2", "error occurred 2", "this passed 2")
    # assert_test(False, "error occurred 3")
    # assert_test(False, "just some testing 4", "error occurred 4")
    # assert_test(False, "just some testing 5", "error occurred 5", "this passed 5")


if __name__ == '__main__':
    main()



from thompcoutils.string_utils import list_to_string


class FailedTestException(Exception):
    pass


def test(method, params, expected_value=None, expected_exception=None):
    method_name = str(method).split(" ")[1]
    print("starting {}({})...".format(method_name, list_to_string(params)), end="")
    if expected_exception is None:
        if expected_value is None:
            method(*params)
        elif method(*params) == expected_value:
            print("passed")
        else:
            str1 = "FAILED EXPECTED VALUE {}({})".format(method_name, list_to_string(params))
            raise FailedTestException(str1)
    else:
        try:
            if method(*params) == expected_value:
                str1 = "FAILED EXCEPTION {}({})".format(method_name, list_to_string(params))
                raise FailedTestException(str1)
        except expected_exception:
            print("passed")



class UnhandledBoolType(Exception):
    pass


def to_bool(v):
    valid_responses = ["yes", "y", "true", "1", "valid"]
    invalid_responses = ["no", "n", "false", "0", "invalid"]
    try:
        v = v.lower()
    except AttributeError:
        v = str(v)

    if v in valid_responses:
        return True
    elif v in invalid_responses:
        return False
    else:
        raise UnhandledBoolType()


def list_to_string(s):
    str1 = ""

    # traverse in the string
    for ele in s:
        str1 += str(ele)

        # return string
    return str1


if __name__ == "__main__":
    import thompcoutils.test_utils as test_utils

    test_utils.test(to_bool, ["valid"], expected_value=True)
    test_utils.test(to_bool, ["invalid"], expected_value=False)
    test_utils.test(to_bool, ["Yes"], expected_value=True)
    test_utils.test(to_bool, ["No"], expected_value=False)
    test_utils.test(to_bool, ["y"], expected_value=True)
    test_utils.test(to_bool, ["n"], expected_value=False)
    test_utils.test(to_bool, ["True"], expected_value=True)
    test_utils.test(to_bool, ["FALSE"], expected_value=False)
    test_utils.test(to_bool, [1], expected_value=True)
    test_utils.test(to_bool, [0], expected_value=False)
    test_utils.test(to_bool, ["dd"], expected_value=False, expected_exception=UnhandledBoolType)

    try:
        to_bool(1)
    except UnhandledBoolType:
        raise Exception("Should not fail")
    try:
        to_bool(2)
    except UnhandledBoolType:
        "pased"


import zipfile
import os
from shutil import copyfile
from enum import Enum
import gzip
from pathlib import Path
from thompcoutils import log_utils
import logging


class ZipException(Exception):
    pass


class ZipType(Enum):
    ZIP = 1
    GZIP = 2


def touch(file_name):
    Path(file_name).touch()


def zip_file(from_file, to_file=None, overwrite=False, method=ZipType.ZIP):
    if to_file is None:
        if method == ZipType.ZIP:
            extension = ".zip"
        elif method == ZipType.GZIP:
            extension = ".gz"
        else:
            raise ZipException("Method {} is not supported".format(str(method)))
        to_file = os.path.splitext(from_file)[0] + extension
    if os.path.isfile(to_file) and not overwrite:
        raise ZipException("{} exists".format(to_file))
    if method == ZipType.ZIP:
        # noinspection PyBroadException
        try:
            compression = zipfile.ZIP_DEFLATED
        except Exception:
            compression = zipfile.ZIP_STORED
        zf = zipfile.ZipFile(to_file, mode='w')
        try:
            zf.write(from_file, compress_type=compression)
        finally:
            zf.close()
    elif method == ZipType.GZIP:
        with open(from_file, 'rb') as orig_file:
            with gzip.open(to_file, 'wb') as zipped_file:
                zipped_file.writelines(orig_file)
    else:
        raise ZipException("Method {} is not supported".format(str(method)))
    return to_file


def _test_zip(original_file, target_file=None, method=None, extension=None):
    logger = log_utils.get_logger()
    if target_file is None:
        zipped_file = os.path.splitext(original_file)[0] + extension
        if os.path.isfile(zipped_file):
            os.remove(zipped_file)
        completed_file = zip_file(original_file, method=method)
        if zipped_file != completed_file:
            raise Exception("{} is not the same as {}".format(zipped_file, completed_file))
        logger.debug("{} zipped to {}".format(original_file, completed_file))
        try:
            zip_file(original_file, method=method)
            raise Exception("Should not be able to overwrite existing file {}".format(zipped_file))
        except ZipException:
            pass
        completed_file = zip_file(original_file, overwrite=True, method=method)
        if zipped_file != completed_file:
            raise Exception("{} is not the same as {}".format(zipped_file, completed_file))
        os.remove(zipped_file)
    else:
        if os.path.isfile(target_file):
            os.remove(target_file)
        completed_file = zip_file(original_file, target_file, method=method)
        if target_file != completed_file:
            raise Exception("{} is not the same as {}".format(target_file, completed_file))
        logger.debug("{} zipped to {}".format(original_file, completed_file))
        try:
            zip_file(original_file, target_file, method=method)
            raise Exception("Should not be able to overwrite existing file {}".format(target_file))
        except ZipException:
            pass
        completed_file = zip_file(original_file, target_file, overwrite=True, method=method)
        if target_file != completed_file:
            raise Exception("{} is not the same as {}".format(target_file, completed_file))
        os.remove(target_file)


def main():
    logger = log_utils.get_logger()
    for method in ZipType:
        logger.debug("Testing {}".format(method))
        if method == ZipType.ZIP:
            extension = ".zip"
        elif method == ZipType.GZIP:
            extension = ".gz"
        else:
            raise Exception("Untested extension {}".format(method))
        original_file = os.path.basename(__file__)
        zipped_file = os.path.splitext(original_file)[0] + "1" + extension
        _test_zip(original_file, method=method, extension=extension)
        _test_zip(original_file, zipped_file, method=method, extension=extension)
        tmp_file = os.path.join("/tmp", original_file)
        zipped_file = os.path.splitext(tmp_file)[0] + "1" + extension
        copyfile(original_file, tmp_file)
        _test_zip(tmp_file, method=method, extension=extension)
        _test_zip(tmp_file, zipped_file, method=method, extension=extension)
        os.remove(tmp_file)


if __name__ == "__main__":
    log_configuration_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'logging.conf')
    logging.config.fileConfig(log_configuration_file)
    main()
